"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _isArray = _interopRequireDefault(require("lodash/isArray"));

var _Valid = _interopRequireDefault(require("./Valid"));

var _SelectMode = _interopRequireDefault(require("../_statics/SelectMode"));

var _Util = _interopRequireDefault(require("./Util"));

/**
 * @file Calculation vendor
 * @author liangxiaojun(liangxiaojun@derbysoft.com)
 */
function pageSize(pageSize, pageSizes, defaultValue) {
  if (pageSize) {
    return pageSize;
  }

  if (pageSizes && pageSizes.length > 0) {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = pageSizes[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var item = _step.value;

        if (item > 0) {
          return item;
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator["return"] != null) {
          _iterator["return"]();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  }

  return defaultValue;
}

function displayIndexByScrollTop(data, listHeight, itemHeight) {
  var scrollTop = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;
  var buffer = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0;

  if (!data || !listHeight || !itemHeight) {
    return {
      start: 0,
      stop: 0,
      startWithBuffer: 0,
      stopWithBuffer: 0
    };
  }

  var len = data.length,
      start = Math.floor(scrollTop / itemHeight),
      stop = start + Math.ceil(listHeight / itemHeight);
  return {
    start: _Valid["default"].range(start, 0, len - 1),
    stop: _Valid["default"].range(stop, 0, len - 1),
    startWithBuffer: _Valid["default"].range(start - buffer, 0, len - 1),
    stopWithBuffer: _Valid["default"].range(stop + buffer, 0, len - 1)
  };
}

function displayIndexByScrollTopMulColumns(data, listHeight, itemHeight, column) {
  var scrollTop = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0;
  var buffer = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 0;

  if (!data || !listHeight || !itemHeight) {
    return {
      start: 0,
      stop: 0,
      startWithBuffer: 0,
      stopWithBuffer: 0
    };
  }

  var len = data.length,
      start = Math.floor(scrollTop / itemHeight) * column,
      stop = start + Math.ceil(listHeight / itemHeight) * column;
  return {
    start: _Valid["default"].range(start, 0, len - 1),
    stop: _Valid["default"].range(stop, 0, len - 1),
    startWithBuffer: _Valid["default"].range(_Valid["default"].range(start, 0, len - 1) - buffer, 0, len - 1),
    stopWithBuffer: _Valid["default"].range(stop + buffer, 0, len - 1)
  };
}

function getInitValue(props) {
  if (!props) {
    return;
  }

  var value = props.value,
      selectMode = props.selectMode;

  if (!selectMode) {
    return;
  }

  if (value != null) {
    return value;
  }

  switch (selectMode) {
    case _SelectMode["default"].MULTI_SELECT:
      return [];

    case _SelectMode["default"].SINGLE_SELECT:
      return null;

    default:
      return value;
  }
}

function getMultiSelectItemIndex(item, value, _ref) {
  var selectMode = _ref.selectMode,
      valueField = _ref.valueField,
      displayField = _ref.displayField;

  if (selectMode !== _SelectMode["default"].MULTI_SELECT || !item || !value) {
    return -1;
  }

  if (!(0, _isArray["default"])(value)) {
    return -1;
  }

  return value.findIndex(function (valueItem) {
    return _Util["default"].isValueEqual(valueItem, item, valueField, displayField);
  });
}

function isItemChecked(item, value, _ref2) {
  var selectMode = _ref2.selectMode,
      valueField = _ref2.valueField,
      displayField = _ref2.displayField;

  if (!item || value == null) {
    return false;
  } // Single Selection


  if (selectMode === _SelectMode["default"].SINGLE_SELECT) {
    return _Util["default"].isValueEqual(value, item, valueField, displayField);
  } // Multiple Selection


  if (!(0, _isArray["default"])(value) || value.length < 1) {
    return false;
  }

  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = value[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var v = _step2.value;

      if (_Util["default"].isValueEqual(v, item, valueField, displayField)) {
        return true;
      }
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2["return"] != null) {
        _iterator2["return"]();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  return false;
}

function isItemIndeterminate(node, value, _ref3) {
  var valueField = _ref3.valueField,
      displayField = _ref3.displayField;

  if (!node || !node.children || node.children.length < 1 || !value || !value.length || value.length < 1) {
    return false;
  }

  var total = 0,
      count = 0;

  _Util["default"].preOrderTraverse(node, function (nodeItem) {
    total++;

    if (value.findIndex(function (item) {
      return _Util["default"].getValueByValueField(item, valueField, displayField) === _Util["default"].getValueByValueField(nodeItem, valueField, displayField);
    }) > -1) {
      count++;
    }
  });

  return count > 0 && total !== count;
}

function filterLocalAutoCompleteData(filter, props) {
  var data = props.data,
      minFilterLength = props.minFilterLength;

  if (!filter || filter.length < minFilterLength) {
    return data;
  }

  var valueField = props.valueField,
      displayField = props.displayField,
      renderer = props.renderer,
      filterCallback = props.filterCallback;

  if (filterCallback) {
    return filterCallback(filter, data);
  }

  return data && data.filter(function (item) {
    if (!item) {
      return false;
    }

    if (renderer) {
      return renderer(item).toString().toUpperCase().includes(filter.toUpperCase());
    }

    if ((0, _typeof2["default"])(item) === 'object') {
      var itemDisplay = _Util["default"].getTextByDisplayField(item, displayField, valueField);

      if (itemDisplay) {
        return itemDisplay.toString().toUpperCase().includes(filter.toUpperCase());
      }
    }

    return item.toString().toUpperCase().includes(filter.toUpperCase());
  });
}

var _default = {
  pageSize: pageSize,
  displayIndexByScrollTop: displayIndexByScrollTop,
  getInitValue: getInitValue,
  getMultiSelectItemIndex: getMultiSelectItemIndex,
  displayIndexByScrollTopMulColumns: displayIndexByScrollTopMulColumns,
  isItemChecked: isItemChecked,
  isItemIndeterminate: isItemIndeterminate,
  filterLocalAutoCompleteData: filterLocalAutoCompleteData
};
exports["default"] = _default;