"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _addClass = _interopRequireDefault(require("dom-helpers/addClass"));

var _removeClass = _interopRequireDefault(require("dom-helpers/removeClass"));

var _hasClass = _interopRequireDefault(require("dom-helpers/hasClass"));

var _Event = _interopRequireDefault(require("./Event"));

/**
 * @file PopManagement vendor
 * @author liangxiaojun(liangxiaojun@derbysoft.com)
 */
var list = [];
var initial = false;

function setBodyLock() {
  var body = document.querySelector('body');

  if (!(0, _hasClass["default"])(body, 'dialog-modal-lock')) {
    (0, _addClass["default"])(body, 'dialog-modal-lock');
  }
}

;

function setBodyUnlock() {
  var body = document.querySelector('body');

  if ((0, _hasClass["default"])(body, 'dialog-modal-lock')) {
    (0, _removeClass["default"])(body, 'dialog-modal-lock');
  }
}

;

function keyDownHandler(e) {
  if (e.keyCode === 27) {
    // esc
    var item = list && list.length > 0 ? list[list.length - 1] : null;

    if (item && item.popContext) {
      var onRequestClose = item.popContext.props.onRequestClose;
      onRequestClose && onRequestClose(e);
    }
  }
}

function addKeyDownEvent() {
  if (!initial) {
    _Event["default"].addEvent(document, 'keydown', keyDownHandler);

    initial = true;
  }
}

function getIndex(popContext) {
  return popContext && list && list.length > 0 ? list.findIndex(function (item) {
    return item && item.popContext && item.popContext == popContext;
  }) : -1;
}

function has(popContext) {
  return getIndex(popContext) !== -1;
}

function push(popContext, config) {
  addKeyDownEvent();

  if (!has(popContext)) {
    list.push({
      popContext: popContext,
      config: config
    }); // if it is Dialog, set body lock

    if (config && config.shouldLockBody) {
      setBodyLock();
    }
  }
}

function pop(popContext) {
  if (!popContext) {
    return;
  }

  var index = getIndex(popContext);

  if (index === -1) {
    return;
  }

  list.splice(index, 1); // if there is no shouldLockBody pop in list, remove body lock

  if (!list || list.length < 1 || !list.some(function (item) {
    return item && item.config && item.config.shouldLockBody;
  })) {
    setBodyUnlock();
  }
}

var _default = {
  getIndex: getIndex,
  has: has,
  push: push,
  pop: pop
};
exports["default"] = _default;