"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _Position = _interopRequireDefault(require("../_statics/Position"));

var _Dom = _interopRequireDefault(require("./Dom"));

/**
 * @file PositionPopCalculation vendor
 * @author liangxiaojun(liangxiaojun@derbysoft.com)
 */
function horizontalCenter(popupEl) {
  return (window.innerWidth - popupEl.offsetWidth) / 2;
}

function verticalMiddle(popupEl) {
  return (window.innerHeight - popupEl.offsetHeight) / 2;
}

function getStyle(parentEl, popupEl, position) {
  if (!popupEl) {
    return;
  }

  var result = {};

  switch (position) {
    case _Position["default"].TOP_LEFT:
      {
        result.left = 0;
        result.top = 0;
        break;
      }

    case _Position["default"].TOP:
      {
        result.left = horizontalCenter(popupEl);
        result.top = 0;
        break;
      }

    case _Position["default"].TOP_RIGHT:
      {
        result.right = 0;
        result.top = 0;
        break;
      }

    case _Position["default"].BOTTOM_LEFT:
      {
        result.left = 0;
        result.bottom = 0;
        break;
      }

    case _Position["default"].BOTTOM:
      {
        result.left = horizontalCenter(popupEl);
        result.bottom = 0;
        break;
      }

    case _Position["default"].BOTTOM_RIGHT:
      {
        result.right = 0;
        result.bottom = 0;
        break;
      }

    case _Position["default"].LEFT_TOP:
      {
        result.left = 0;
        result.top = 0;
        break;
      }

    case _Position["default"].LEFT:
      {
        result.left = 0;
        result.top = verticalMiddle(popupEl);
        break;
      }

    case _Position["default"].LEFT_BOTTOM:
      {
        result.left = 0;
        result.bottom = 0;
        break;
      }

    case _Position["default"].RIGHT_TOP:
      {
        result.right = 0;
        result.top = 0;
        break;
      }

    case _Position["default"].RIGHT:
      {
        result.right = 0;
        result.top = verticalMiddle(popupEl);
        break;
      }

    case _Position["default"].RIGHT_BOTTOM:
      {
        result.right = 0;
        result.bottom = 0;
        break;
      }

    case _Position["default"].CENTER:
      {
        result.left = horizontalCenter(popupEl);
        result.top = verticalMiddle(popupEl);
        break;
      }
  }

  return result;
}

function setStyle(parentEl, popupEl, position) {
  var style = getStyle(parentEl, popupEl, position);

  if (!style) {
    return;
  }

  popupEl.style.left = popupEl.style.right = popupEl.style.top = popupEl.style.bottom = null;

  for (var key in style) {
    popupEl.style[key] = style[key] + 'px';
  }
}

var _default = {
  getStyle: getStyle,
  setStyle: setStyle
};
exports["default"] = _default;