"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _classnames = _interopRequireDefault(require("classnames"));

var _HorizontalAlign = _interopRequireDefault(require("../_statics/HorizontalAlign"));

var _VirtualRoot = _interopRequireDefault(require("../_statics/VirtualRoot"));

var _TableFragment = _interopRequireDefault(require("../_statics/TableFragment"));

var _Util = _interopRequireDefault(require("../_vendors/Util"));

var _Valid = _interopRequireDefault(require("../_vendors/Valid"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function calcSpan(type, column, data, colIndex, rowIndex) {
  var span = column["".concat(type, "Span")];
  return span && typeof span === 'function' ? span(data, colIndex, rowIndex) : +span;
}

function getColumnsWithSpan(type, columns, data, rowIndex) {
  if (!type) {
    return columns.map(function (column) {
      return {
        column: column,
        span: 1
      };
    });
  }

  var result = [];
  var spanFlag = 0;

  for (var colIndex = 0, len = columns.length; colIndex < len; colIndex++) {
    if (spanFlag > 1) {
      spanFlag--;
      continue;
    }

    var column = columns[colIndex],
        span = calcSpan(type, columns[colIndex], data, colIndex, rowIndex);

    if (span && span > 1) {
      spanFlag = span;
    }

    result.push({
      column: column,
      span: span
    });
  }

  return result;
}

function sortTableData(data, sorting, sortFunc) {
  if (!data) {
    return [];
  }

  if (!sorting) {
    return data;
  }

  var copyData = data.slice();

  if (sortFunc) {
    copyData = sortFunc(copyData, sorting);
  } else {
    copyData.sort(function (a, b) {
      if (!isNaN(a[sorting.prop]) && !isNaN(b[sorting.prop])) {
        return (Number(a[sorting.prop]) - Number(b[sorting.prop])) * sorting.type;
      } else {
        return (a[sorting.prop] + '').localeCompare(b[sorting.prop] + '') * sorting.type;
      }
    });
  }

  return copyData;
}

function hasRenderer(columns, fragment) {
  return columns && columns.length > 0 && fragment ? columns.some(function (column) {
    return column && column["".concat(fragment, "Renderer")];
  }) : false;
}

function hasHeadRenderer(columnsGroup) {
  return columnsGroup && columnsGroup.length > 0 ? columnsGroup.some(function (columns) {
    return hasRenderer(columns, _TableFragment["default"].HEAD);
  }) : false;
}

function hasFixedColumn(columns, fixed) {
  return columns && fixed ? columns.some(function (column) {
    return column && column.fixed === fixed;
  }) : false;
}

function getDataByPagination(data, isPaginated, pagination) {
  if (!data || data.length < 1) {
    return [];
  }

  if (!isPaginated || !pagination) {
    return data;
  }

  return data.slice(pagination.page * pagination.pageSize, (pagination.page + 1) * pagination.pageSize).filter(function (item) {
    return item && !item.disabled;
  });
}

function indexOfNodeInValue(node, value, idProp) {
  if (!node || !value) {
    return -1;
  }

  var index = value.findIndex(function (item) {
    return idProp in item && idProp in node && item[idProp] === node[idProp];
  });

  if (index < 0) {
    index = value.indexOf(node);
  }

  return index;
}

function isNodeChecked(node, value, idProp) {
  return indexOfNodeInValue(node, value, idProp) >= 0;
}

function isSelectAllChecked(data, value, idProp) {
  var _Util$preOrderTravers;

  var total = 0,
      count = 0;

  _Util["default"].preOrderTraverse((_Util$preOrderTravers = {}, (0, _defineProperty2["default"])(_Util$preOrderTravers, _VirtualRoot["default"], true), (0, _defineProperty2["default"])(_Util$preOrderTravers, "children", data), _Util$preOrderTravers), function (node) {
    if (node && !node.disabled && !(_VirtualRoot["default"] in node)) {
      total++;

      if (isNodeChecked(node, value, idProp)) {
        count++;
      }
    }
  });

  return {
    checked: total > 0 && count === total,
    indeterminate: count > 0 && total !== count
  };
}

function handleSelect(node, value, idProp, isSelectRecursive) {
  if (!node || node.disabled || !value) {
    return value;
  }

  if (!isNodeChecked(node, value, idProp)) {
    value.push(node);
  }

  if (!isSelectRecursive || !node.children || node.children.length < 1) {
    return value;
  }

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = node.children[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var item = _step.value;
      handleSelect(item, value, idProp);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return value;
}

function handleDeselect(node, value, idProp, isSelectRecursive) {
  if (!node || node.disabled || !value) {
    return value;
  }

  var index = indexOfNodeInValue(node, value, idProp);

  if (index > -1) {
    value.splice(index, 1);
  }

  if (!isSelectRecursive || !node.children || node.children.length < 1) {
    return value;
  }

  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = node.children[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var item = _step2.value;
      handleDeselect(item, value, idProp);
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2["return"] != null) {
        _iterator2["return"]();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  return value;
}

function formatValue(value, data, idProp) {
  var _Util$postOrderTraver;

  var result = [];

  _Util["default"].postOrderTraverse((_Util$postOrderTraver = {}, (0, _defineProperty2["default"])(_Util$postOrderTraver, _VirtualRoot["default"], true), (0, _defineProperty2["default"])(_Util$postOrderTraver, "children", data), _Util$postOrderTraver), function (node) {
    if (!(_VirtualRoot["default"] in node)) {
      if (!node.children || node.children.length < 1) {
        if (isNodeChecked(node, value, idProp)) {
          result.push(node);
        }
      } else {
        if (node.children.every(function (child) {
          return isNodeChecked(child, value, idProp);
        })) {
          result.push(node);
        }
      }
    }
  });

  return result;
}

function handleSelectAll(data, value, idProp) {
  if (!data || data.length < 1) {
    return value;
  }

  data.forEach(function (node) {
    return handleSelect(node, value, idProp, true);
  });
  return value;
}
/**
 * separate columns to left, center and right, and check has fixed columns or not
 * @param columns
 * @returns {{hasFixedLeftColumn: (Array|boolean), sortedColumns: *[], hasFixedRightColumn: (Array|boolean)}|*}
 */


function sortColumns(columns) {
  var _result;

  if (!columns || columns.length < 1) {
    return columns;
  }

  var result = (_result = {}, (0, _defineProperty2["default"])(_result, _HorizontalAlign["default"].LEFT, []), (0, _defineProperty2["default"])(_result, _HorizontalAlign["default"].CENTER, []), (0, _defineProperty2["default"])(_result, _HorizontalAlign["default"].RIGHT, []), _result); // separate root column to left, right or center

  columns.forEach(function (column) {
    return column && result[column.fixed || _HorizontalAlign["default"].CENTER].push(column);
  }); // traverse all children nodes in left and right, update "fixed" property inheriting from their parent

  _Util["default"].preOrderTraverse({
    children: result[_HorizontalAlign["default"].LEFT]
  }, function (node) {
    return node.fixed = _HorizontalAlign["default"].LEFT;
  });

  _Util["default"].preOrderTraverse({
    children: result[_HorizontalAlign["default"].RIGHT]
  }, function (node) {
    return node.fixed = _HorizontalAlign["default"].RIGHT;
  });

  return {
    sortedColumns: [].concat((0, _toConsumableArray2["default"])(result[_HorizontalAlign["default"].LEFT]), (0, _toConsumableArray2["default"])(result[_HorizontalAlign["default"].CENTER]), (0, _toConsumableArray2["default"])(result[_HorizontalAlign["default"].RIGHT])),
    hasFixedLeftColumn: result[_HorizontalAlign["default"].LEFT] && result[_HorizontalAlign["default"].LEFT].length > 0,
    hasFixedRightColumn: result[_HorizontalAlign["default"].RIGHT] && result[_HorizontalAlign["default"].RIGHT].length > 0
  };
}

function handleFixedColumnsClassName(columns) {
  if (!columns || columns.length < 1) {
    return columns;
  }

  return columns.map(function (column) {
    return column && column.fixed && column.fixed !== _HorizontalAlign["default"].CENTER ? _objectSpread({}, column, {
      headClassName: (0, _classnames["default"])(column.headClassName, 'table-fixed-column'),
      bodyClassName: (0, _classnames["default"])(column.bodyClassName, 'table-fixed-column'),
      footClassName: (0, _classnames["default"])(column.footClassName, 'table-fixed-column')
    }) : column;
  });
}
/**
 * get the deepest first child of columns
 * @param columns
 * @returns {null|*}
 */


function getFirstColumn(columns) {
  if (!columns || columns.length < 1) {
    return null;
  }

  if (columns[0] && (!columns[0].children || columns[0].children.length < 1)) {
    return columns[0];
  }

  return getFirstColumn(columns[0].children);
}
/**
 * calculate the row span and column span
 * @param node
 * @param maxDepth
 * @param depth
 * @returns {number|*}
 */


function formatColumnsSpan(node, maxDepth) {
  var depth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : -1;

  if (!node) {
    return 0;
  }

  var hasChildren = node.children && node.children.length > 0;
  var colSpan = 0;

  if (hasChildren) {
    node.children.forEach(function (childNode) {
      return childNode && (colSpan += formatColumnsSpan(childNode, maxDepth, depth + 1));
    }); // column span

    if (colSpan > 1) {
      node.colSpan = colSpan;
    }
  } else {
    // row span
    var rowSpan = maxDepth - depth;

    if (rowSpan > 1) {
      node.rowSpan = rowSpan;
    }
  }

  if (node[_VirtualRoot["default"]]) {
    return node.children;
  }

  if (hasChildren) {
    return colSpan;
  }

  return 1;
}
/**
 * get head columns
 * transform the columns data (tree construction) to html tr-th row format
 * @param columns
 * @returns {Array|*}
 */


function getHeadColumns(columns) {
  var _formatColumnsSpan;

  if (!columns || columns.length < 1) {
    return columns;
  } // calculate the max depth


  var maxDepth = 0;

  _Util["default"].postOrderTraverse({
    children: columns
  }, function (node, depth) {
    if (depth > maxDepth) {
      maxDepth = depth;
    }
  }); // add row span and column span to column node


  var formatedColumns = formatColumnsSpan((_formatColumnsSpan = {}, (0, _defineProperty2["default"])(_formatColumnsSpan, _VirtualRoot["default"], true), (0, _defineProperty2["default"])(_formatColumnsSpan, "children", columns), _formatColumnsSpan), maxDepth);

  if (!formatedColumns) {
    return columns;
  }

  var result = [];

  _Util["default"].postOrderTraverse({
    children: formatedColumns
  }, function (node, depth) {
    var index = depth - 1;

    if (index >= 0) {
      if (!result[index]) {
        result[index] = [];
      }

      result[index].push(node);
    }
  });

  return result;
}

function getBodyColumns(columns) {
  if (!columns || columns.length < 1) {
    return columns;
  }

  var result = [];

  _Util["default"].postOrderTraverse({
    children: columns
  }, function (node) {
    if (node && (!node.children || node.children.length < 1)) {
      result.push(node);
    }
  });

  return result;
}

function getFixedHeadColumns(columns, fixed) {
  return columns ? columns.map(function (row) {
    return row ? row.filter(function (column) {
      return column && column.fixed === fixed;
    }) : row;
  }) : columns;
}

function getFixedBodyColumns(columns, fixed) {
  return columns ? columns.filter(function (column) {
    return column && column.fixed === fixed;
  }) : columns;
}

function recursiveSelectChildren(node) {
  var value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var idProp = arguments.length > 2 ? arguments[2] : undefined;

  if (!node) {
    return value;
  }

  _Util["default"].preOrderTraverse(node, function (child) {
    if (!isNodeChecked(child, value, idProp)) {
      value.push(child);
    }
  });

  return value;
}

function needCollapseButtonSpacing(tableData) {
  return tableData && tableData.some(function (rowData) {
    return rowData && rowData.children && rowData.children.length > 0;
  });
}

function getPageSizeValue(pageSize, pageSizeValueField) {
  return (0, _typeof2["default"])(pageSize) === 'object' ? pageSize[pageSizeValueField] : pageSize;
}

function handleNoWrap(fragmentNoWrap, columnNoWrap, _ref) {
  var data = _ref.data,
      colIndex = _ref.colIndex,
      rowIndex = _ref.rowIndex,
      tableData = _ref.tableData;
  return !!((typeof fragmentNoWrap === 'function' ? fragmentNoWrap(data, colIndex, rowIndex, tableData) : fragmentNoWrap) || (typeof columnNoWrap === 'function' ? columnNoWrap(data, colIndex, rowIndex, tableData) : columnNoWrap));
}

function handlePage(page, pageSize, data) {
  if (!data || data.length < 1 || !pageSize || page < 0) {
    return 0;
  }

  return _Valid["default"].range(page, 0, Math.ceil(data.length / pageSize) - 1);
}

var _default = {
  calcSpan: calcSpan,
  getColumnsWithSpan: getColumnsWithSpan,
  sortTableData: sortTableData,
  hasRenderer: hasRenderer,
  hasHeadRenderer: hasHeadRenderer,
  hasFixedColumn: hasFixedColumn,
  getDataByPagination: getDataByPagination,
  indexOfNodeInValue: indexOfNodeInValue,
  isNodeChecked: isNodeChecked,
  isSelectAllChecked: isSelectAllChecked,
  handleSelect: handleSelect,
  handleDeselect: handleDeselect,
  formatValue: formatValue,
  handleSelectAll: handleSelectAll,
  sortColumns: sortColumns,
  handleFixedColumnsClassName: handleFixedColumnsClassName,
  getFirstColumn: getFirstColumn,
  getHeadColumns: getHeadColumns,
  getBodyColumns: getBodyColumns,
  getFixedHeadColumns: getFixedHeadColumns,
  getFixedBodyColumns: getFixedBodyColumns,
  recursiveSelectChildren: recursiveSelectChildren,
  needCollapseButtonSpacing: needCollapseButtonSpacing,
  getPageSizeValue: getPageSizeValue,
  handleNoWrap: handleNoWrap,
  handlePage: handlePage
};
exports["default"] = _default;