"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _sum = _interopRequireDefault(require("lodash/sum"));

var _hasClass = _interopRequireDefault(require("dom-helpers/hasClass"));

var _addClass = _interopRequireDefault(require("dom-helpers/addClass"));

var _removeClass = _interopRequireDefault(require("dom-helpers/removeClass"));

var _TableFragment = _interopRequireDefault(require("../_statics/TableFragment"));

var _HorizontalAlign = _interopRequireDefault(require("../_statics/HorizontalAlign"));

var _Direction = _interopRequireDefault(require("../_statics/Direction"));

var _ScrollBar = _interopRequireDefault(require("./ScrollBar"));

var _Valid = _interopRequireDefault(require("./Valid"));

/**
 * @file TableLayout vendor
 * @author liangxiaojun(liangxiaojun@derbysoft.com)
 */

/**
 * whether a table has a fixed head or a fixed foot or fixed left columns or fixed right columns
 * @param props
 * @param columns
 * @returns {boolean}
 */
function hasFixed(props, config) {
  if (props && (props.isHeadFixed || props.isFootFixed)) {
    return true;
  }

  if (config && (config.hasFixedLeftColumn || config.hasFixedRightColumn)) {
    return true;
  }

  return false;
}
/**
 * calculate each head, body and foot column width
 * @returns {null|{[p: string]: *|*}}
 */


function getColumnsWidth(tableEl) {
  var _ref;

  if (!tableEl) {
    return null;
  }

  var bodyTr = tableEl.querySelector('tbody tr'),
      footTr = tableEl.querySelector('tfoot tr');
  return _ref = {}, (0, _defineProperty2["default"])(_ref, _TableFragment["default"].BODY, bodyTr ? [].map.call(bodyTr.querySelectorAll('td'), function (td) {
    return td ? td.offsetWidth : 0;
  }) : []), (0, _defineProperty2["default"])(_ref, _TableFragment["default"].FOOT, footTr ? [].map.call(footTr.querySelectorAll('td'), function (td) {
    return td ? td.offsetWidth : 0;
  }) : []), _ref;
}
/**
 * calculate each head, body and foot row height
 * @returns {null|{[p: string]: *|*}}
 */


function getRowsHeight(tableEl) {
  var _ref2;

  if (!tableEl) {
    return null;
  }

  return _ref2 = {}, (0, _defineProperty2["default"])(_ref2, _TableFragment["default"].HEAD, [].map.call(tableEl.querySelectorAll('thead tr'), function (el) {
    return el.offsetHeight;
  })), (0, _defineProperty2["default"])(_ref2, _TableFragment["default"].BODY, [].map.call(tableEl.querySelectorAll('tbody tr'), function (el) {
    return el.offsetHeight;
  })), (0, _defineProperty2["default"])(_ref2, _TableFragment["default"].FOOT, [].map.call(tableEl.querySelectorAll('tfoot tr'), function (el) {
    return el.offsetHeight;
  })), _ref2;
}
/**
 * calculate table body scroller height
 * @param headHeight
 * @param footHeight
 * @returns {{height: string}}
 */


function getbodyScollerHeight(headHeight, footHeight) {
  return "calc(100%".concat(headHeight ? " - ".concat((0, _sum["default"])(headHeight), "px") : '').concat(footHeight ? " - ".concat((0, _sum["default"])(footHeight), "px") : '', ")");
}
/**
 * mask center body table to hide its fixed head, fixed foot and fixed columns
 * @param wrapperEl
 * @param tableEl
 * @param fixedHeadHeight
 * @param fixedFootHeight
 */


function maskCenterBody(wrapperEl, rawTableEl, fixedHeadHeight, fixedFootHeight) {
  var tableEl = wrapperEl.querySelector('.table-content-center .scroll-table-body table'),
      maskEl = wrapperEl.querySelector('.table-content-center .scroll-table-body-mask');

  if (rawTableEl && tableEl && maskEl) {
    maskEl.style.height = "".concat(_Valid["default"].range(tableEl.offsetHeight - fixedHeadHeight - fixedFootHeight - 1, 0), "px");
    tableEl.style.marginTop = "".concat(-fixedHeadHeight - 1, "px");

    if (maskEl.offsetWidth !== rawTableEl.offsetWidth) {
      maskEl.style.width = "".concat(rawTableEl.offsetWidth, "px");
    }
  }
}
/**
 * fix center head and foot table width
 * @param wrapperEl
 */


function fixCenterTableWidth(wrapperEl) {
  var centerBody = wrapperEl.querySelector('.table-content-center .scroll-table-body-scroller table');

  if (!centerBody) {
    return;
  }

  var centerHead = wrapperEl.querySelector('.table-content-center .scroll-table-head-scroller table');

  if (centerHead) {
    centerHead.style.width = "".concat(centerBody.offsetWidth, "px");
  }

  var centerFoot = wrapperEl.querySelector('.table-content-center .scroll-table-foot-scroller table');

  if (centerFoot) {
    centerFoot.style.width = "".concat(centerBody.offsetWidth, "px");
  }
}
/**
 * fix head, body or foot columns width according to the center body table columns width
 * @param wrapperEl
 * @param columnsWidth
 * @param fixed
 * @param fragment
 * @param selector
 */


function fixTableFragmentColumnsWidth(wrapperEl, columnsWidth, fixed, fragment, selector) {
  var el = wrapperEl.querySelector("".concat(selector, " .scroll-table-").concat(fragment, "-scroller"));

  if (el) {
    var cols = el.querySelectorAll('col');

    if (cols) {
      if (fixed === _HorizontalAlign["default"].RIGHT) {
        [].forEach.call(cols, function (el, index) {
          if (el) {
            el.style.width = "".concat(columnsWidth[columnsWidth.length - (cols.length - index)], "px");
          }
        });
      } else {
        [].forEach.call(cols, function (el, index) {
          if (el) {
            el.style.width = "".concat(columnsWidth[index], "px");
          }
        });
      }
    }
  }
}
/**
 * fix table columns width, including head, body and foot
 * @param wrapperEl
 * @param columnsWidth
 * @param fixed
 * @param props
 */


function fixTableColumnsWidth(wrapperEl, columnsWidth, fixed, props) {
  if (!wrapperEl) {
    return;
  }

  var selector = ".table-content-".concat(fixed || 'center');
  /**
   * head
   */

  if (props && props.isHeadFixed) {
    fixTableFragmentColumnsWidth(wrapperEl, columnsWidth[_TableFragment["default"].BODY], fixed, _TableFragment["default"].HEAD, selector);
  }
  /**
   * body
   */
  // if (fixed) {


  fixTableFragmentColumnsWidth(wrapperEl, columnsWidth[_TableFragment["default"].BODY], fixed, _TableFragment["default"].BODY, selector); // }

  /**
   * foot
   */

  if (props && props.isHeadFixed) {
    fixTableFragmentColumnsWidth(wrapperEl, columnsWidth[_TableFragment["default"].FOOT], fixed, _TableFragment["default"].FOOT, selector);
  }
}
/**
 * fix head, body or foot rows height according to the center body table rows height
 * @param wrapperEl
 * @param rowsHeight
 * @param fixed
 * @param fragment
 * @param selector
 */


function fixTableFragmentRowsHeight(wrapperEl, rowsHeight, fixed, fragment, subFragment, selector) {
  var el = wrapperEl.querySelector("".concat(selector, " .scroll-table-").concat(fragment, " t").concat(subFragment));

  if (el) {
    var trs = el.querySelectorAll('tr');

    if (trs) {
      [].forEach.call(trs, function (el, index) {
        if (el) {
          el.style.height = "".concat(rowsHeight[subFragment][index], "px");
        }
      });
    }
  }
}
/**
 * fix table rows height, including head, body and foot
 * @param wrapperEl
 * @param rowsHeight
 * @param fixed
 * @param props
 */


function fixTableRowsHeight(wrapperEl, rowsHeight, fixed, props) {
  if (!wrapperEl) {
    return;
  }

  var selector = ".table-content-".concat(fixed || 'center');
  /**
   * head
   */

  if (props && props.isHeadFixed) {
    fixTableFragmentRowsHeight(wrapperEl, rowsHeight, fixed, _TableFragment["default"].HEAD, _TableFragment["default"].HEAD, selector);
  }
  /**
   * body
   */


  if (fixed) {
    fixTableFragmentRowsHeight(wrapperEl, rowsHeight, fixed, _TableFragment["default"].BODY, _TableFragment["default"].HEAD, selector);
    fixTableFragmentRowsHeight(wrapperEl, rowsHeight, fixed, _TableFragment["default"].BODY, _TableFragment["default"].BODY, selector);
    fixTableFragmentRowsHeight(wrapperEl, rowsHeight, fixed, _TableFragment["default"].BODY, _TableFragment["default"].FOOT, selector);
  }
  /**
   * foot
   */


  if (props && props.isHeadFixed) {
    fixTableFragmentRowsHeight(wrapperEl, rowsHeight, fixed, _TableFragment["default"].FOOT, _TableFragment["default"].FOOT, selector);
  }
}
/**
 * fix horizontal scroll style, including fixed head, fixed foot, and fixed columns table body
 * @param el
 * @param shouldScroll
 * @param size
 */


function fixHorizontalScrollStyle(el, shouldScroll) {
  var size = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _ScrollBar["default"].getSize(_Direction["default"].HORIZONTAL);

  if (!el) {
    return;
  }

  if (!shouldScroll) {
    el.style.marginBottom = 0;
    el.style.paddingBottom = 0;
  } else if (size && size > 0) {
    el.style.marginBottom = "".concat(-size, "px");
    el.style.paddingBottom = 0;
  } else {
    el.style.marginBottom = '-20px';
    el.style.paddingBottom = '20px';
  }
}
/* eslint-disable complexity */

/**
 * fix horizontal scroll
 * @param wrapperEl
 * @param rawTableEl
 * @param props
 */


function fixTableHorizontalScroll(wrapperEl, rawTableEl, props) {
  if (!wrapperEl || !rawTableEl) {
    return;
  } // no need to fix the scroll problem in mac


  var horizontalScrollBarSize = _ScrollBar["default"].getSize(_Direction["default"].HORIZONTAL);

  if (!horizontalScrollBarSize || horizontalScrollBarSize <= 0) {
    return;
  }

  var shouldScroll = rawTableEl.offsetWidth > wrapperEl.offsetWidth;
  /**
   * center head scroller
   */

  if (props && props.isHeadFixed) {
    var centerHeadScroller = wrapperEl.querySelector('.table-content-center .scroll-table-head-scroller');

    if (centerHeadScroller) {
      fixHorizontalScrollStyle(centerHeadScroller, true, horizontalScrollBarSize);
    }
  }
  /**
   * center foot scroller
   */


  if (props && props.isFootFixed) {
    var centerFoot = wrapperEl.querySelector('.table-content-center .scroll-table-foot-scroller');

    if (centerFoot) {
      fixHorizontalScrollStyle(centerFoot, true, horizontalScrollBarSize);
    }
  }
  /**
   * left body scroller
   */


  var leftBody = wrapperEl.querySelector('.table-content-left .scroll-table-body-scroller');

  if (leftBody) {
    fixHorizontalScrollStyle(leftBody, shouldScroll, horizontalScrollBarSize);
  }
  /**
   * left body mask
   */


  var leftBodyMask = wrapperEl.querySelector('.table-content-left .scroll-table-body-mask');

  if (leftBodyMask) {
    leftBodyMask.style.paddingBottom = shouldScroll ? "".concat(horizontalScrollBarSize, "px") : 0;
  }
  /**
   * left foot
   */


  var leftFoot = wrapperEl.querySelector('.table-content-left .scroll-table-foot');

  if (leftFoot) {
    leftFoot.style.marginTop = shouldScroll ? "".concat(horizontalScrollBarSize, "px") : 0;
  }
  /**
   * right body scroller
   */


  var rightBody = wrapperEl.querySelector('.table-content-right .scroll-table-body-scroller');

  if (rightBody) {
    fixHorizontalScrollStyle(rightBody, shouldScroll, horizontalScrollBarSize);
  }
  /**
   * right body mask
   */


  var rightBodyMask = wrapperEl.querySelector('.table-content-right .scroll-table-body-mask');

  if (rightBodyMask) {
    rightBodyMask.style.paddingBottom = shouldScroll ? "".concat(horizontalScrollBarSize, "px") : 0;
  }
  /**
   * right foot
   */


  var rightFoot = wrapperEl.querySelector('.table-content-right .scroll-table-foot');

  if (rightFoot) {
    rightFoot.style.marginTop = shouldScroll ? "".concat(horizontalScrollBarSize, "px") : 0;
  }
}
/**
 * fix vertical scroll
 * @param wrapperEl
 * @param props
 */


function fixTableVerticalScroll(wrapperEl, props) {
  if (!wrapperEl) {
    return;
  }

  var verticalScrollBarSize = _ScrollBar["default"].getSize(_Direction["default"].VERTICAL);
  /**
   * fix center head width
   */
  // if (props && props.isHeadFixed && verticalScrollBarSize && verticalScrollBarSize > 0) {
  //     const centerHead = wrapperEl.querySelector('.table-content-center .scroll-table-head-scroller');
  //     if (centerHead) {
  //         centerHead.style.width = `calc(100% - ${verticalScrollBarSize}px)`;
  //     }
  // }

  /**
   * fix center foot width
   */
  // if (props && props.isFootFixed && verticalScrollBarSize && verticalScrollBarSize > 0) {
  //     const centerFoot = wrapperEl.querySelector('.table-content-center .scroll-table-foot-scroller');
  //     if (centerFoot) {
  //         centerFoot.style.width = `calc(100% - ${verticalScrollBarSize}px)`;
  //     }
  // }

  /**
   * fix left body horizontal scroll bar
   */


  var leftBodyScroller = wrapperEl.querySelector('.table-content-left .scroll-table-body-scroller');

  if (leftBodyScroller) {
    // has vertical scroll bar width, like windows
    if (verticalScrollBarSize && verticalScrollBarSize > 0) {
      // only fix when has vertical scroll
      if (leftBodyScroller.offsetHeight < leftBodyScroller.scrollHeight) {
        leftBodyScroller.style.paddingRight = 0;
        leftBodyScroller.style.marginRight = "-".concat(verticalScrollBarSize, "px");
      }
    } // no vertical scroll bar width, like mac
    else {
        var leftBodyTable = leftBodyScroller.querySelector('table');

        if (leftBodyTable) {
          var restWidth = wrapperEl.offsetWidth - leftBodyTable.offsetWidth;
          leftBodyScroller.style.marginRight = "-".concat(restWidth, "px");
          leftBodyScroller.style.paddingRight = "".concat(restWidth, "px");
        }
      }
  }
}
/**
 * fix table layout
 * @param wrapperEl
 * @param rawTableEl
 * @param props
 */


function fixLayout(wrapperEl, rawTableEl, props) {
  if (!wrapperEl || !rawTableEl) {
    return;
  }

  var columnsWidth = getColumnsWidth(rawTableEl),
      rowsHeight = getRowsHeight(rawTableEl),
      fixedHeadHeight = (0, _sum["default"])(rowsHeight[_TableFragment["default"].HEAD]) || 0,
      fixedFootHeight = (0, _sum["default"])(rowsHeight[_TableFragment["default"].FOOT]) || 0;
  /**
   * fix horizontal scroll style
   */

  fixTableHorizontalScroll(wrapperEl, rawTableEl, props);
  /**
   * center
   */

  if (props && (props.isHeadFixed || props.isFootFixed)) {
    maskCenterBody(wrapperEl, rawTableEl, fixedHeadHeight, fixedFootHeight);
  }

  fixCenterTableWidth(wrapperEl);
  fixTableColumnsWidth(wrapperEl, columnsWidth, null, props);
  fixTableRowsHeight(wrapperEl, rowsHeight, null, props);
  /**
   * left
   */

  fixTableColumnsWidth(wrapperEl, columnsWidth, _HorizontalAlign["default"].LEFT, props);
  fixTableRowsHeight(wrapperEl, rowsHeight, _HorizontalAlign["default"].LEFT, props);
  /**
   * right
   */

  fixTableColumnsWidth(wrapperEl, columnsWidth, _HorizontalAlign["default"].RIGHT, props);
  fixTableRowsHeight(wrapperEl, rowsHeight, _HorizontalAlign["default"].RIGHT, props);
  /**
   * fix vertical scroll style
   */

  fixTableVerticalScroll(wrapperEl, props);
}
/**
 * update scroll class names when horizontal scrolling
 * @param wrapperEl
 * @param scrollerEl
 */


function updateHorizontalScrollClassNames(wrapperEl, scrollerEl) {
  if (!wrapperEl || !scrollerEl) {
    return;
  }

  var verticalScrollBarSize = scrollerEl.offsetHeight < scrollerEl.scrollHeight ? _ScrollBar["default"].getSize(_Direction["default"].VERTICAL) : 0;
  (0, _removeClass["default"])(wrapperEl, 'scroll-left');
  (0, _removeClass["default"])(wrapperEl, 'scroll-center');
  (0, _removeClass["default"])(wrapperEl, 'scroll-right'); // no scroll

  if (scrollerEl.scrollWidth + verticalScrollBarSize === scrollerEl.offsetWidth) {
    return;
  }

  if (scrollerEl.scrollLeft === 0 && !(0, _hasClass["default"])(wrapperEl, 'scroll-left')) {
    (0, _removeClass["default"])(wrapperEl, 'scroll-center');
    (0, _removeClass["default"])(wrapperEl, 'scroll-right');
    (0, _addClass["default"])(wrapperEl, 'scroll-left');
  } else if (scrollerEl.scrollLeft === scrollerEl.scrollWidth - scrollerEl.offsetWidth + verticalScrollBarSize && !(0, _hasClass["default"])(wrapperEl, 'scroll-right')) {
    (0, _removeClass["default"])(wrapperEl, 'scroll-left');
    (0, _removeClass["default"])(wrapperEl, 'scroll-center');
    (0, _addClass["default"])(wrapperEl, 'scroll-right');
  } else if (!(0, _hasClass["default"])(wrapperEl, 'scroll-center')) {
    (0, _removeClass["default"])(wrapperEl, 'scroll-left');
    (0, _removeClass["default"])(wrapperEl, 'scroll-right');
    (0, _addClass["default"])(wrapperEl, 'scroll-center');
  }
}
/**
 * return scroll styles
 * @param scroll
 * @returns {{verticalScrollStyle: null, horizontalScrollStyle: null}}
 */


function getScrollerStyle(scroll) {
  var result = {
    horizontalScrollStyle: null,
    verticalScrollStyle: null
  };

  if (!scroll) {
    return result;
  }

  if ('width' in scroll) {
    if (!result.horizontalScrollStyle) {
      result.horizontalScrollStyle = {};
    }

    result.horizontalScrollStyle.minWidth = scroll.width;
  }

  if ('height' in scroll) {
    if (!result.verticalScrollStyle) {
      result.verticalScrollStyle = {};
    }

    result.verticalScrollStyle.height = scroll.height;
  }

  if ('maxHeight' in scroll) {
    if (!result.verticalScrollStyle) {
      result.verticalScrollStyle = {};
    }

    result.verticalScrollStyle.maxHeight = scroll.maxHeight;
  }

  return result;
}

var _default = {
  hasFixed: hasFixed,
  getColumnsWidth: getColumnsWidth,
  getRowsHeight: getRowsHeight,
  getbodyScollerHeight: getbodyScollerHeight,
  fixLayout: fixLayout,
  updateHorizontalScrollClassNames: updateHorizontalScrollClassNames,
  getScrollerStyle: getScrollerStyle
};
exports["default"] = _default;