"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _moment = _interopRequireDefault(require("moment"));

/**
 * @file Utils vendor
 * @author liangxiaojun(liangxiaojun@derbysoft.com)
 */
function isEnableLocalStorage() {
  try {
    return 'localStorage' in window && window['localStorage'] !== null;
  } catch (e) {
    return false;
  }
}

function isEnableSessionStorage() {
  try {
    return 'sessionStorage' in window && window['sessionStorage'] !== null;
  } catch (e) {
    return false;
  }
}

function isEnableCookieAndStorage() {
  return navigator.cookieEnabled && isEnableLocalStorage() && isEnableSessionStorage();
}

function formatCapitalize(value) {
  return value ? value.charAt(0).toUpperCase() + value.substring(1).toLowerCase() : value;
}

function value2Timestamp(value, format) {
  var defaultValue = new Date().getTime();

  if (typeof value === 'number') {
    return new Date(value).toString() === 'Invalid Date' ? defaultValue : value;
  } else if (typeof value === 'string') {
    var date = (0, _moment["default"])(value, format);
    return date.isValid() ? date.valueOf() : defaultValue;
  } else if (_moment["default"].isDate(value)) {
    var _date = (0, _moment["default"])(value);

    return _date.isValid() ? _date.valueOf() : defaultValue;
  }

  return defaultValue;
}

function value2Moment(value, format) {
  var defaultValue = (0, _moment["default"])();

  if (typeof value === 'string') {
    var date = (0, _moment["default"])(value, format);
    return date.isValid() ? date : defaultValue;
  } else {
    var _date2 = (0, _moment["default"])(value);

    return _date2.isValid() ? _date2 : defaultValue;
  }
}

function MonthDays(year) {
  // debugger
  var dateArray = [];

  for (var i = 0; i < 12; i++) {
    switch (i + 1) {
      case 1:
      case 3:
      case 5:
      case 7:
      case 8:
      case 10:
      case 12:
        dateArray.push(31);
        break;

      case 4:
      case 6:
      case 9:
      case 11:
        dateArray.push(30);
        break;

      case 2:
        if ((0, _moment["default"])(year + '-02-29', 'YYYY-MM-DD', true).isValid()) {
          dateArray.push(29);
        } else {
          dateArray.push(28);
        }

        break;

      default:
        break;
    }
  }

  return dateArray;
}

function enumerateValue(enumerate) {
  if (!enumerate) {
    return;
  }

  return Object.keys(enumerate).map(function (key) {
    return enumerate[key];
  });
}

function tree(data, callback) {
  var loop = function loop(children, level) {
    children.forEach(function (item, index) {
      var pos = level + '-' + (1 + index);

      if (item.children) {
        loop(item.children, pos);
      }

      callback(item, index, pos);
    });
  };

  loop(data, 1);
}

function getValueByValueField(data) {
  var valueField = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'value';
  var displayField = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'text';

  if (data == null) {
    return;
  }

  if ((0, _typeof2["default"])(data) === 'object') {
    return data[valueField] == null ? data[displayField] : data[valueField];
  }

  return data;
}

function getTextByDisplayField(data) {
  var displayField = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'text';
  var valueField = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'value';

  if (data == null) {
    return '';
  }

  if ((0, _typeof2["default"])(data) === 'object') {
    return data[displayField] == null ? data[valueField] : data[displayField];
  }

  return data;
}

function isValueEqual(data1, data2) {
  var valueField = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'value';
  var displayField = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'text';
  return getValueByValueField(data1, valueField, displayField) == getValueByValueField(data2, valueField, displayField);
}

function genIndexArray(len) {
  if (!len || isNaN(len) || len < 1) {
    return [];
  }

  var result = [];

  for (var i = 0; i < len; i++) {
    result[i] = i;
  }

  return result;
}

function getDiag(a, b) {
  return Math.sqrt(a * a + b * b);
}

function reorder(data, startIndex, endIndex) {
  if (!data || !(startIndex in data) || !(startIndex in data)) {
    return;
  }

  var _data$splice = data.splice(startIndex, 1),
      _data$splice2 = (0, _slicedToArray2["default"])(_data$splice, 1),
      removed = _data$splice2[0];

  data.splice(endIndex, 0, removed);
}

function preOrderTraverse(node, callback) {
  var depth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  var parent = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

  if (callback(node, depth, parent) === false) {
    return false;
  }

  if (node.children && node.children.length > 0) {
    for (var i = 0, len = node.children.length; i < len; i++) {
      if (preOrderTraverse(node.children[i], callback, depth + 1, node) === false) {
        break;
      }
    }
  }
}

function postOrderTraverse(node, callback) {
  var depth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  var parent = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

  if (node.children && node.children.length > 0) {
    for (var i = 0, len = node.children.length; i < len; i++) {
      if (postOrderTraverse(node.children[i], callback, depth + 1, node) === false) {
        break;
      }
    }
  }

  if (callback(node, depth, parent) === false) {
    return false;
  }
}

var _default = {
  isEnableLocalStorage: isEnableLocalStorage,
  isEnableSessionStorage: isEnableSessionStorage,
  isEnableCookieAndStorage: isEnableCookieAndStorage,
  formatCapitalize: formatCapitalize,
  value2Timestamp: value2Timestamp,
  value2Moment: value2Moment,
  MonthDays: MonthDays,
  enumerateValue: enumerateValue,
  tree: tree,
  getValueByValueField: getValueByValueField,
  getTextByDisplayField: getTextByDisplayField,
  isValueEqual: isValueEqual,
  genIndexArray: genIndexArray,
  getDiag: getDiag,
  reorder: reorder,
  preOrderTraverse: preOrderTraverse,
  postOrderTraverse: postOrderTraverse
};
exports["default"] = _default;